package de.bsvrz.buv.plugin.darstellung.actions;

import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.jface.action.StatusLineContributionItem;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;

import de.bsvrz.buv.plugin.darstellung.model.Darstellung;

/**
 * Basisklasse fr Status Items, die Informationen zu einer Darstellung
 * anzeigen.
 * 
 * <p>
 * Das Status Item meldet sich auf den aktuellen Workbench Part an und wird nur
 * aktiviert, wenn der aktuelle Part ber {@link GraphicalViewer},
 * {@link ZoomManager} und {@link Darstellung} verfgt.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id: DarstellungStatus.java 26023 2010-09-01 16:54:32Z Schumann $
 */
public abstract class DarstellungStatus extends StatusLineContributionItem {

	private final class IPartListenerImplementation implements IPartListener {

		@Override
		public void partOpened(final IWorkbenchPart part) {
			updateStatus(part);
		}

		@Override
		public void partDeactivated(final IWorkbenchPart part) {
			updateStatus(part);
		}

		@Override
		public void partClosed(final IWorkbenchPart part) {
			updateStatus(part);
		}

		@Override
		public void partBroughtToTop(final IWorkbenchPart part) {
			updateStatus(part);
		}

		@Override
		public void partActivated(final IWorkbenchPart part) {
			updateStatus(part);
		}
	}

	private final IWorkbenchPage page;
	private final IPartListenerImplementation listener;
	private IWorkbenchPart currentPart;
	private GraphicalViewer viewer;
	private ZoomManager zoomManager;
	private Darstellung darstellung;

	/**
	 * Initialisietr das Status Item.
	 * 
	 * @param page
	 * @param id
	 * @param charWidth
	 */
	public DarstellungStatus(final IWorkbenchPage page, final String id,
			final int charWidth) {
		super(id, charWidth);
		this.page = page;

		listener = new IPartListenerImplementation();
		page.addPartListener(listener);
	}

	private void updateStatus(final IWorkbenchPart part) {
		final IEditorPart activeEditor = page.getActiveEditor();
		if (currentPart != null && currentPart != activeEditor) {
			deactivate();
			viewer = null;
			zoomManager = null;
			darstellung = null;
			currentPart = null;
		}

		if (activeEditor != null) {
			final GraphicalViewer v = (GraphicalViewer) page.getActiveEditor()
					.getAdapter(GraphicalViewer.class);
			final ZoomManager zm = (ZoomManager) page.getActiveEditor()
					.getAdapter(ZoomManager.class);
			final Darstellung d = (Darstellung) page.getActiveEditor()
					.getAdapter(Darstellung.class);

			if (v != null && d != null && zm != null) {
				currentPart = part;
				darstellung = d;
				zoomManager = zm;
				viewer = v;
				activate();
			}
		}
	}

	/**
	 * Gibt den Graphical Viewer des aktuellen Workbench Parts zurck.
	 * 
	 * @return der Viewer.
	 */
	protected GraphicalViewer getViewer() {
		return viewer;
	}

	/**
	 * Gibt den Zoom anager des aktuellen Workbench Parts zurck.
	 * 
	 * @return der Zoom Manager.
	 */
	protected ZoomManager getZoomManager() {
		return zoomManager;
	}

	/**
	 * Gibt die Darstellung des aktuellen Workbench Parts zurck.
	 * 
	 * @return der Darstellung.
	 */
	protected Darstellung getDarstellung() {
		return darstellung;
	}

	/**
	 * Wird aufgerufen, wenn der aktuelle Workbench Part fr dieses Status Item
	 * verwendet werden kann.
	 * 
	 * <p>
	 * Hier sollten Listener angemeldet werden.
	 */
	protected abstract void activate();

	/**
	 * Wird aufgerufen, wenn der aktuelle Workbench Part fr dieses Status Item
	 * <em>nicht</em> verwendet werden kann.
	 * 
	 * <p>
	 * Hier sollten Listener wieder abgemeldet werden.
	 */
	protected abstract void deactivate();

	@Override
	public void dispose() {
		page.removePartListener(listener);

		super.dispose();
	}

}
