package de.bsvrz.buv.plugin.darstellung.util;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionPoint;

import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.MassstaeblicheDarstellung;
import de.bsvrz.buv.plugin.dobj.util.Projektion;

/**
 * Projekt bei der die Lngen- und Breitengrade als x- und y-Koordinaten direkt
 * verwendet werden. Bei der quadratischen Plattkarte wird um Zerrungen in der
 * Darstullung zu verringern, abhnghngig von der Breite ein Faktor zur
 * Korrektur der Lnge eingesetzt.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class QuadratischePlattkarteProjektion implements Projektion {

	/**
	 * WGS-84-Koordinaten im Datenkatalog besitzen eine Genauigkeit von sechs
	 * Nachkommastellen, deswegen werden die Lngen- und Breitengrade mit
	 * 1.000.000 multipliziert.
	 */
	private static final int DEFAULT_SKALIERUNG = 1000000;

	private double minimaleGeographischeLaenge;
	private double maximaleGeographischeBreite;
	private double laengenKorrektur;

	/**
	 * Gibt den Faktor fr die Korrektur der geografischen Breite, abhngig von
	 * der geografischen Lnge zurck.
	 * 
	 * @return den Faktor der Lngenkorrektur.
	 */
	public double getLaengenKorrektur() {
		return laengenKorrektur;
	}

	public void setDarstellung(final Darstellung darstellung) {
		Assert.isLegal(darstellung instanceof MassstaeblicheDarstellung);

		final MassstaeblicheDarstellung md = (MassstaeblicheDarstellung) darstellung;

		minimaleGeographischeLaenge = md.getMinimaleGeographischeLaenge();
		maximaleGeographischeBreite = md.getMaximaleGeographischeBreite();

		final double minimaleGeographischeBreite = md
				.getMinimaleGeographischeBreite();
		double breiteInRad = minimaleGeographischeBreite
				+ (maximaleGeographischeBreite - minimaleGeographischeBreite)
				/ 2;
		breiteInRad = breiteInRad / 360 * 2 * Math.PI;
		laengenKorrektur = Math.cos(breiteInRad);
	}

	@Override
	public Point umrechnenModellNachWelt(final Point modell) {
		double dx = modell.preciseX();
		double dy = modell.preciseY();

		// Skalierung rckgngig machen
		dx /= DEFAULT_SKALIERUNG;
		dy /= DEFAULT_SKALIERUNG;

		// Lngenkorrektur rckgngig machen
		dx /= getLaengenKorrektur();

		// Verschiebung nach (0,0) rckgngig machen
		dx += minimaleGeographischeLaenge;
		dy = maximaleGeographischeBreite - dy;

		return new PrecisionPoint(dx, dy);
	}

	@Override
	public Point umrechnenWeltNachModel(final Point welt) {
		double dx = welt.preciseX();
		double dy = welt.preciseY();

		// Verschieben der oberen linken Ecke der Karte nach (0,0)
		dx -= minimaleGeographischeLaenge;
		dy = maximaleGeographischeBreite - dy;

		// Einfache Lngenkorrektur
		dx *= getLaengenKorrektur();

		// Skalierung der double-Werte in int-Werte
		dx *= DEFAULT_SKALIERUNG;
		dy *= DEFAULT_SKALIERUNG;

		return new PrecisionPoint(dx, dy);
	}

}
