package de.bsvrz.buv.plugin.darstellung.wizards;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.bitctrl.lib.eclipse.emf.viewers.EmfLabelProvider;

import de.bsvrz.buv.rw.bitctrl.eclipse.modell.emf.EMFModellEinstellungen;
import de.bsvrz.buv.rw.bitctrl.eclipse.viewers.EinstellungenContentProvider;

/**
 * Basisklasse fr eine Wizard Page, die die notwendigen Informationen (Objekte
 * und Verzeichnis) fr den Export bestimmt.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public abstract class AbstractExportWizardPage extends WizardPage {

	private final EClass eClass;
	private final EMFModellEinstellungen<?> einstellungen;

	private TreeViewer viewer;
	private Text directoryText;
	private Button fileChooseButton;
	private ISelection preSelection;

	/**
	 * Initialisiert die Page.
	 */
	protected AbstractExportWizardPage(final String pageName,
			final EClass eClass, final EMFModellEinstellungen<?> einstellungen) {
		super(pageName);

		this.eClass = eClass;
		this.einstellungen = einstellungen;

		setPageComplete(false);
	}

	@Override
	public void createControl(final Composite parent) {
		final Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout(3, false));

		createViewer(container);
		createDirectory(container);

		viewer.getControl().setFocus();
		setControl(container);
	}

	private void createViewer(final Composite container) {
		final Label label = new Label(container, SWT.NONE);
		label.setText(eClass.getName());
		label.setLayoutData(GridDataFactory.swtDefaults().align(SWT.LEFT,
				SWT.TOP).create());

		viewer = new TreeViewer(container, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.BORDER);
		viewer.getControl().setLayoutData(
				GridDataFactory.fillDefaults().grab(true, true).span(2, 1)
						.create());
		viewer.setContentProvider(new EinstellungenContentProvider());
		viewer.setLabelProvider(new EmfLabelProvider());
		viewer.setInput(einstellungen);
		viewer.setSelection(getPreSelection(), true);
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(final SelectionChangedEvent event) {
				updatePageComplete();
			}

		});
	}

	private void createDirectory(final Composite container) {
		Label label;
		label = new Label(container, SWT.NONE);
		label.setText("Zielverzeichnis");

		directoryText = new Text(container, SWT.BORDER);
		directoryText.setLayoutData(GridDataFactory.swtDefaults().align(
				SWT.FILL, SWT.CENTER).grab(true, false).create());
		directoryText.addModifyListener(new ModifyListener() {

			@Override
			public void modifyText(final ModifyEvent e) {
				updatePageComplete();
			}

		});

		fileChooseButton = new Button(container, SWT.PUSH);
		fileChooseButton.setText("Auswhlen");
		fileChooseButton.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(final SelectionEvent e) {
				final DirectoryDialog dlg = new DirectoryDialog(getShell());
				dlg.setText(getTitle());
				dlg.setMessage(getDescription());
				final String selected = dlg.open();
				if (selected != null) {
					directoryText.setText(selected);
					updatePageComplete();
				}
			}

		});
	}

	private void updatePageComplete() {
		if (viewer.getSelection().isEmpty()) {
			setErrorMessage("Sie mssen mindestens ein zu exportierendes Objekt auswhlen.");
			setPageComplete(false);
		} else if (directoryText.getText().isEmpty()) {
			setErrorMessage("Sie mssen das Verzeichnis angeben, in das die ausgewhlten Objekte exportiert werden sollen.");
			setPageComplete(false);
		} else {
			setErrorMessage(null);
			setPageComplete(true);
		}
	}

	/**
	 * Gibt das Zielverzeichnis fr den Export zurck.
	 */
	public String getDirectory() {
		return directoryText.getText();
	}

	/**
	 * Gibt die zu exportierten Objekte zurck.
	 */
	public ISelection getSelection() {
		return viewer.getSelection();
	}

	/**
	 * Gibt die Vorauswahl fr die zu exportierenden Objekte zurck.
	 */
	public ISelection getPreSelection() {
		return preSelection;
	}

	/**
	 * Legt eine Vorauswahl fr die zu exportierenden Objekte fest.
	 */
	public void setPreSelection(final ISelection preSelection) {
		this.preSelection = preSelection;
	}

}
