package de.bsvrz.buv.plugin.dobj.editparts;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import de.bsvrz.buv.plugin.dobj.model.DoModel;

/**
 * Basisklasse fr die Factory eines Viewer, der {@link DoModel}s anzeigen soll.
 * 
 * Diese Factory arbeitet nur mit Modellobjekten die von der Klasse
 * {@link DoModel} abgeleitet sind. Fr die konrekte Darstellung im Editor,
 * einer Karte usw. muss diese Factory um die zustzlichen Modellobjekte (z. B.
 * <code>MassstaeblicheDarstellung</code>) erweitert werden.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class DoEditPartFactory implements EditPartFactory {

	/**
	 * Erzeugt das Edit Part und weist ihm das Modellobjekt zu.
	 * 
	 * <p>
	 * Modellobjekte vom Typ {@link DoModel} behandelt die Methdode selbst, alle
	 * anderen Objekte werden an {@link #doCreateEditPart(EditPart, Object)}
	 * delegiert.
	 * 
	 * @see #doCreateEditPart(EditPart, Object)
	 */
	@Override
	public final EditPart createEditPart(final EditPart context,
			final Object model) {
		final EditPart editPart;
		if (model instanceof DoModel) {
			final DoModel doModel = (DoModel) model;
			editPart = doModel.getDoTyp().createController();
		} else {
			editPart = doCreateEditPart(context, model);
		}

		if (editPart.getModel() == null) {
			editPart.setModel(model);
		}

		return editPart;
	}

	/**
	 * Erzeugt den Edit Part zu einem Modellobjekt, das kein {@link DoModel}
	 * ist.
	 * 
	 * <p>
	 * Darf in abgeleiteten Klassen nur erweitert werden. Der Aufruf der
	 * Supermethode muss dabei als letztes erfolgen.
	 * 
	 * @param context
	 *            der bergeordnete Edit Part.
	 * @param model
	 *            das Modellobjekt zu dem ein Edit Part gesucht ist.
	 * @return der Edit Part fr das Modellobjekt, niemals <code>null</code>.
	 */
	protected EditPart doCreateEditPart(final EditPart context,
			final Object model) {
		throw new IllegalArgumentException("Unbekanntes Modellobjekt: "
				+ model.getClass());
	}

}
