package de.bsvrz.buv.plugin.dobj.graph;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.bitctrl.graph.AbstractGraph;
import com.bitctrl.graph.Bogen;
import com.bitctrl.graph.Knoten;

import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdAeusseresStrassenSegment.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.AeusseresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Netz;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.NetzBestandTeil;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Reprsentiert einen Graph aus Straenknoten und ueren Straensegmenten.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class NetzGraph extends AbstractGraph {

	private final Netz netz;
	private final Map<StrassenKnoten, Knoten> knotenListe = new HashMap<StrassenKnoten, Knoten>();
	private final Map<AeusseresStrassenSegment, Bogen> bogenListe = new HashMap<AeusseresStrassenSegment, Bogen>();

	/**
	 * Erzeugt einen Netzgraph.
	 */
	public NetzGraph(final Netz netz) {
		this.netz = netz;
		init(netz);
	}

	/**
	 * Gibt das gekapselte netz zurck.
	 */
	public Netz getNetz() {
		return netz;
	}

	private void init(final Netz n) {
		final long starttime = System.currentTimeMillis();

		knotenListe.clear();
		bogenListe.clear();

		for (final NetzBestandTeil nbt : n.getNetzBestandTeile()) {
			if (nbt instanceof Netz) {
				// Verschachteltes Netz analysieren
				init(n);
			} else if (nbt instanceof AeusseresStrassenSegment) {
				// Neuer Bogen im Netz analysieren
				final AeusseresStrassenSegment aess = (AeusseresStrassenSegment) nbt;
				final Daten datum = aess.getKdAeusseresStrassenSegment()
						.getDatum();
				if (datum != null) {
					// Anfangsknoten des neuen Bogens registrieren
					final StrassenKnoten vonKnoten = datum.getVonKnoten();
					NetzKnoten anfangsKnoten = null;
					if (vonKnoten != null) {
						if (!knotenListe.containsKey(vonKnoten)) {
							anfangsKnoten = new NetzKnoten(vonKnoten);
							knotenListe.put(vonKnoten, anfangsKnoten);
						} else {
							anfangsKnoten = (NetzKnoten) knotenListe
									.get(vonKnoten);
						}
					}

					// Endknoten des neuen Bogens registrieren
					final StrassenKnoten nachKnoten = datum.getNachKnoten();
					NetzKnoten endKnoten = null;
					if (nachKnoten != null) {
						if (!knotenListe.containsKey(nachKnoten)) {
							endKnoten = new NetzKnoten(nachKnoten);
							knotenListe.put(nachKnoten, endKnoten);
						} else {
							endKnoten = (NetzKnoten) knotenListe
									.get(nachKnoten);
						}
					}

					// Neuen Bogen registrieren
					bogenListe.put(aess, new NetzBogen(aess, anfangsKnoten,
							endKnoten));
				} else {
					Debug.getLogger()
							.warning(
									"ueres Straensegment ist nicht vollstndig konfiguriert und wird deshalb ignoriert:",
									aess);
				}
			} else {
				// Unbekanter Netzbestandteil gefunden
				throw new IllegalStateException("Unbekanntes NetzBestandTeil: "
						+ nbt);
			}
		}

		for (final Knoten knoten : getKnoten()) {
			((NetzKnoten) knoten).init(this);
		}

		final long endtime = System.currentTimeMillis();
		final long duration = (endtime - starttime) / 1000;
		Debug.getLogger().info(
				"Netzanalsyse in " + duration + " Sekunden beendet.");
	}

	/**
	 * Gibt den Netzknoten zu Straenknoten zurck.
	 * 
	 * @param strassenKnoten
	 *            der Straenknoten zu dem der Netzknoten gesucht wird.
	 * @return der Netzknoten oder <code>null</code>, wenn der Straenknoten
	 *         nicht zum Netz gehrt.
	 */
	public NetzKnoten getKnoten(final StrassenKnoten strassenKnoten) {
		return (NetzKnoten) knotenListe.get(strassenKnoten);
	}

	@Override
	public Set<Knoten> getKnoten() {
		return Collections.unmodifiableSet(new HashSet<Knoten>(knotenListe
				.values()));
	}

	/**
	 * Gibt den Netzbogen zu einem ueren Straensegment zurck.
	 * 
	 * @param aeusseresStrassenSegment
	 *            das uere Straensegment zu dem der Netzbogen gesucht wird.
	 * @return der Netzbogen oder <code>null</code>, wenn das uere
	 *         Straensegment nicht zum Netz gehrt.
	 */
	public NetzBogen getBogen(
			final AeusseresStrassenSegment aeusseresStrassenSegment) {
		return (NetzBogen) bogenListe.get(aeusseresStrassenSegment);
	}

	@Override
	public Collection<Bogen> getBoegen() {
		return Collections.unmodifiableSet(new HashSet<Bogen>(bogenListe
				.values()));
	}

}
