package de.bsvrz.buv.plugin.dobj.graph;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.bitctrl.graph.AbstractKnoten;
import com.bitctrl.graph.Bogen;
import com.bitctrl.util.Tupel;

import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdInneresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.AeusseresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.InneresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Reprsentiert einen Knoten (Straenknoten) im Netzgraph.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class NetzKnoten extends AbstractKnoten {

	private final StrassenKnoten strassenKnoten;
	private final Set<Bogen> eingangsBogenListe = new HashSet<Bogen>();
	private final Set<Bogen> ausgangsBogenListe = new HashSet<Bogen>();
	private final Set<Bogen> inzidentBogenListe = new HashSet<Bogen>();
	private final Map<Tupel<AeusseresStrassenSegment, AeusseresStrassenSegment>, InneresStrassenSegment> innereStrassenSegmente = new HashMap<Tupel<AeusseresStrassenSegment, AeusseresStrassenSegment>, InneresStrassenSegment>();

	/**
	 * Initialisiert den Netzknoten.
	 */
	NetzKnoten(final StrassenKnoten strassenKnoten) {
		this.strassenKnoten = strassenKnoten;
	}

	/**
	 * Gibt den gekapselten Straenknoten zurck.
	 */
	public StrassenKnoten getStrassenKnoten() {
		return strassenKnoten;
	}

	/**
	 * Gibt das innere Straensegment zurck, welches zwei uere
	 * Straensegmente verbindet.
	 * 
	 * @param vonStrassenSegment
	 *            das Straensegment, welches zu diesem Straenknoten hinfhrt.
	 * @param nachStrassenSegment
	 *            das Straensegment, welches von diesem Straenknoten wegfhrt.
	 * @return das innere Straensegment, welches die beiden ueren verbindet
	 *         oder <code>null</code>, wenn es keines gibt.
	 */
	public InneresStrassenSegment getInneresStrassenSegment(
			final AeusseresStrassenSegment vonStrassenSegment,
			final AeusseresStrassenSegment nachStrassenSegment) {
		return innereStrassenSegmente
				.get(new Tupel<AeusseresStrassenSegment, AeusseresStrassenSegment>(
						vonStrassenSegment, nachStrassenSegment));
	}

	void init(final NetzGraph graph) {
		eingangsBogenListe.clear();
		ausgangsBogenListe.clear();
		inzidentBogenListe.clear();

		for (final InneresStrassenSegment iss : strassenKnoten
				.getInnereStrassenSegmente()) {
			final KdInneresStrassenSegment.Daten issDatum = iss
					.getKdInneresStrassenSegment().getDatum();
			if (issDatum == null) {
				Debug.getLogger()
						.warning(
								"Inneres Straensegment ist nicht vollstndig konfiguriert und wird deshalb ignoriert:",
								iss);
				continue;
			}

			final AeusseresStrassenSegment nachStrassenSegment = issDatum
					.getNachStrassenSegment();
			if (nachStrassenSegment != null) {
				final NetzBogen bogen = graph.getBogen(nachStrassenSegment);
				if (bogen == null) {
					// System.err
					// .println("Ausgangsbogen ist nicht im Graph enthalten: "
					// + nachStrassenSegment);
				} else if (equals(bogen.getAnfangsKnoten())) {
					ausgangsBogenListe.add(bogen);
					inzidentBogenListe.add(bogen);
				} else {
					// System.err
					// .println("Anfangsknoten des Ausgangsbogen ist falsch: "
					// + bogen);
				}
			}

			final AeusseresStrassenSegment vonStrassenSegment = issDatum
					.getVonStrassenSegment();
			if (vonStrassenSegment != null) {
				final NetzBogen bogen = graph.getBogen(vonStrassenSegment);
				if (bogen == null) {
					// System.err
					// .println("Eingangsbogen ist nicht im Graph enthalten: "
					// + nachStrassenSegment);
				} else if (equals(bogen.getEndKnoten())) {
					eingangsBogenListe.add(bogen);
					inzidentBogenListe.add(bogen);
				} else {
					System.err
							.println("Endknoten des Eingangsbogen ist falsch: "
									+ bogen);
				}
			}

			if (vonStrassenSegment != null && nachStrassenSegment != null) {
				innereStrassenSegmente
						.put(new Tupel<AeusseresStrassenSegment, AeusseresStrassenSegment>(
								vonStrassenSegment, nachStrassenSegment), iss);
			}

			if (vonStrassenSegment == null && nachStrassenSegment == null) {
				System.err.println("Knoten ohne inzidente Bgen: " + this);
			}
		}
	}

	@Override
	public Iterable<Bogen> ausgangsBogenIterator() {
		return ausgangsBogenListe;
	}

	@Override
	public Iterable<Bogen> eingangsBogenIterator() {
		return eingangsBogenListe;
	}

	@Override
	public Iterable<Bogen> inzidentBogenIterator() {
		return inzidentBogenListe;
	}

	@Override
	public void setStuetzBogen(final Bogen stuetzBogen) {
		if (stuetzBogen == null || stuetzBogen == WURZEL_BOGEN) {
			super.setStuetzBogen(stuetzBogen);
			return;
		}

		if (!equals(stuetzBogen.getEndKnoten())) {
			throw new IllegalArgumentException(
					"Der Sttzbogen muss zu diesem Knotenfhren.");
		}

		final AeusseresStrassenSegment nachStrassenSegment = ((NetzBogen) stuetzBogen)
				.getAeusseresStrassenSegment();

		final NetzKnoten lastKnoten = (NetzKnoten) stuetzBogen
				.getAnfangsKnoten();
		final Bogen lastStuetzBogen = lastKnoten.getStuetzBogen();
		if (lastStuetzBogen == null || lastStuetzBogen == WURZEL_BOGEN) {
			super.setStuetzBogen(stuetzBogen);
			return;
		}

		final AeusseresStrassenSegment vonStrassenSegment = ((NetzBogen) lastStuetzBogen)
				.getAeusseresStrassenSegment();

		if (lastKnoten.getInneresStrassenSegment(vonStrassenSegment,
				nachStrassenSegment) != null) {
			super.setStuetzBogen(stuetzBogen);
		} else {
			System.err.println("Ungltiger Sttzbogen wird ignoriert.");
		}
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		} else if (obj instanceof NetzKnoten) {
			final NetzKnoten other = (NetzKnoten) obj;
			return strassenKnoten.equals(other.strassenKnoten);
		}
		return false;
	}

	@Override
	public int hashCode() {
		return strassenKnoten.hashCode();
	}

	@Override
	public String toString() {
		return strassenKnoten.toString();
	}

}
